# Generated by Django 5.0.6 on 2024-06-21 03:11

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("djpress", "0003_post_menu_order"),
    ]

    operations = [
        migrations.RenameField(
            model_name="category",
            old_name="name",
            new_name="title",
        ),
        migrations.AddField(
            model_name="category",
            name="menu_order",
            field=models.IntegerField(default=0),
        ),
    ]
