# Generated by Django 5.0.6 on 2024-06-12 11:43

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("djpress", "0002_rename_content_type_post_post_type"),
    ]

    operations = [
        migrations.AddField(
            model_name="post",
            name="menu_order",
            field=models.IntegerField(default=0),
        ),
    ]
